//! Error handling for calibre database access.

use thiserror::Error;
use time::error::Parse;

/// Errors from accessing the calibre database.
#[derive(Error, Debug)]
#[error("data store error")]
pub enum DataStoreError {
    /// Found no entries for the query.
    #[error("no results")]
    NoResults(rusqlite::Error),
    /// Error with SQLite.
    #[error("sqlite error")]
    SqliteError(rusqlite::Error),
    /// Error connecting to the database.
    #[error("connection error")]
    ConnectionError(#[from] r2d2::Error),
    /// Error wparsing a datetime from the database.
    #[error("failed to parse datetime")]
    DateTimeError(#[from] Parse),
}

/// Convert an SQLite error into a proper NoResults one if the query
/// returned no rows, return others as is.
impl From<rusqlite::Error> for DataStoreError {
    fn from(error: rusqlite::Error) -> Self {
        match error {
            rusqlite::Error::QueryReturnedNoRows => DataStoreError::NoResults(error),
            _ => DataStoreError::SqliteError(error),
        }
    }
}
